
dnl ---------------------------------------------------------------------------
dnl Check if various types have been typedefed by database header files
dnl ---------------------------------------------------------------------------
AC_DEFUN([MH_CHECK_TYPE],
[
dnl
dnl override existing value of $ac_compile so we use the correct compiler
dnl SHOULD NOT NEED THIS
dnl
ac_compile='$ac_cv_prog_CC conftest.$ac_ext $DB_INCLUDES $CFLAGS $CPPFLAGS -c -o conftest.o 1>&5 2>&5'
for mh_type in $1; do
	AC_MSG_CHECKING(if $mh_type defined)
AC_TRY_COMPILE([#include <${DBINC}>],
[${mh_type} var1;],
  mh_type_found=yes, mh_type_found=no )
	AC_MSG_RESULT($mh_type_found)
	if test "$mh_type_found" = yes; then
	AC_DEFINE_UNQUOTED(${mh_type}_TYPEDEFED)
	fi
done
])dnl

dnl ---------------------------------------------------------------------------
dnl Setup all Oracle stuff
dnl ---------------------------------------------------------------------------
AC_DEFUN([MH_CHECK_ORACLE],
[
DBINC="oci.h"
DBLIB1=""
DBLIB2=""
DBLIB3=""
DBLIB4=""
mh_db_inc_dir=""
mh_db_lib_dir=""
mh_db_name="Oracle"
INCLUDE_DB_MAKEFILE=""
if test -n "$ORACLE_HOME" ; then
	if test -f "$ORACLE_HOME/lib/libclntsh.$SHL"; then
		if test "$need_extra_oracle_libs" = yes; then
			DBLIB1="-L$ORACLE_HOME/lib -lclntsh -lcommon -lcore4 -lnlsrtl3"
		else
			DBLIB1="-L$ORACLE_HOME/lib -lclntsh"
		fi
		mh_db_lib_dir="$ORACLE_HOME/lib"
		nm $ORACLE_HOME/lib/libclntsh.$SHL | grep OCILogon > /dev/null
		if test $? -eq 0; then 
			DBC="ora8"
			DBH="ora8"
			mh_db_inc_dir="$ORACLE_HOME/rdbms/demo"
		else
			DBC="ora"
			DBH="ora"
		fi
		AC_MSG_RESULT(using Oracle client library -lclntsh)
	else
		grep libclntsh $ORACLE_HOME/rdbms/lib/*.mk > /dev/null
		if test $? -eq 0 ; then
			AC_MSG_WARN(This version of Oracle supports a shared library of client code)
			AC_MSG_WARN(used by OCI programs.  The shared library; libclntsh.$SHL in)
			AC_MSG_WARN($ORACLE_HOME/lib has not been built.  See your DBA to build this)
			AC_MSG_WARN(library and then run this configuration script again.)
			AC_MSG_WARN(You will need to include $ORACLE_HOME/lib in your LD_LIBRARY_PATH.)
			AC_MSG_ERROR(configure aborted.)
		else
			if test -f "$ORACLE_HOME/rdbms/lib/oracle.mk" ; then
				INCLUDE_DB_MAKEFILE="include $ORACLE_HOME/rdbms/lib/oracle.mk"
				DBLIB1="-L$ORACLE_HOME/lib $(OCILDLIBS)"
				mh_db_lib_dir="$ORACLE_HOME/lib"
				AC_MSG_RESULT(using $ORACLE_HOME/rdbms/lib/oracle.mk)
			else
				AC_MSG_WARN(Unable to configure Rexx/SQL for Oracle.)
				AC_MSG_WARN(Rexx/SQL needs one of the following files to be available:)
				AC_MSG_WARN()
				AC_MSG_WARN($ORACLE_HOME/lib/libclntsh.$SHL)
				AC_MSG_WARN($ORACLE_HOME/rdbms/lib/oracle.mk)
				AC_MSG_WARN(or)
				AC_MSG_WARN(rules to make libclntsh.$SHL in)
				AC_MSG_WARN($ORACLE_HOME/rdbms/lib/*.mk)
			AC_MSG_ERROR(configure aborted.)
			fi
		fi
	fi
else
	AC_MSG_ERROR(ORACLE_HOME not set cannot continue)
fi
])dnl

dnl ---------------------------------------------------------------------------
dnl Setup all Sybase stuff
dnl ---------------------------------------------------------------------------
AC_DEFUN([MH_CHECK_SYBASE],
[
DBINC="ctpublic.h"
DBLIB1=""
DBLIB2=""
DBLIB3=""
DBLIB4=""
mh_db_inc_dir=""
mh_db_lib_dir=""
mh_db_name="Sybase System 10/11"
AC_MSG_CHECKING(for Sybase headers and libraries)
if test -n "$SYBASE" ; then
	if test "$with_dbincdir" = no; then
		DB_INCLUDES="-I${SYBASE}/include"
		mh_db_inc_dir="$SYBASE/include"
	else
		DB_INCLUDES="-I${with_dbincdir}"
		mh_db_inc_dir="$with_dbincdir"
	fi
	mh_db_lib_dir="$SYBASE/lib"
	mh_syb_libraries="cs tcl comn intl sybtcl insck"
	if test "$with_dblibdir" = no; then
		if test -f "${SYBASE}/lib/libctlib.a"; then
			mh_syb_lib="-lctlib"
		else
			if test -f "${SYBASE}/lib/libct.a"; then
				mh_syb_lib="-lct"
			fi
		fi
		for a_lib in $mh_syb_libraries; do
			if test -f "${SYBASE}/lib/lib${a_lib}.a"; then
				mh_syb_lib="${mh_syb_lib} -l${a_lib}"
			fi
		done
		DB_LIBS="-L${SYBASE}/lib ${mh_syb_lib}"
	else
		if test -f "${with_dblibdir}/libctlib.a"; then
			mh_syb_lib="-lctlib"
		else
			if test -f "${with_dblibdir}/libct.a"; then
				mh_syb_lib="-lct"
			fi
		fi
		for a_lib in $mh_syb_libraries; do
			if test -f "${with_dblibdir}/lib${a_lib}.a"; then
				mh_syb_lib="${mh_syb_lib} -l${a_lib}"
			fi
		done
		DB_LIBS="-L${with_dblibdir} ${mh_syb_lib}"
	fi
	AC_MSG_RESULT(found)
else
	AC_MSG_ERROR(SYBASE not set, cannot continue)
fi
])dnl

dnl ---------------------------------------------------------------------------
dnl Setup all mSQL 2.x stuff
dnl ---------------------------------------------------------------------------
AC_DEFUN([MH_CHECK_MSQL],
[
DBINC="msql.h"
DBLIB1=""
DBLIB2=""
DBLIB3=""
DBLIB4=""
mh_db_inc_dir=""
mh_db_lib_dir=""
mh_db_name="miniSQL 2.x"
AC_MSG_CHECKING(location of mSQL header file)
if test "$with_dbincdir" = no; then
	if test -f "/usr/local/Hughes/include/msql.h"; then
		mh_db_inc_dir="/usr/local/Hughes/include"
	else
		AC_MSG_ERROR(msql.h not found in /usr/local/Hughes/include; you MUST specify directory where msql.h resides using --with-dbincdir=)
	fi
else
	if test -f "$with_dbincdir/msql.h"; then
		mh_db_inc_dir="${with_dbincdir}"
	else
		AC_MSG_ERROR(msql.h not found in $with_dbincdir)
	fi
fi
DB_INCLUDES="-I$mh_db_inc_dir"
AC_MSG_RESULT(found in $mh_db_inc_dir)

AC_MSG_CHECKING(location of mSQL library file)
if test "$with_dblibdir" = no; then
	if test -f "/usr/local/Hughes/lib/libmsql.a"; then
		mh_db_lib_dir="/usr/local/Hughes/lib"
	else
		AC_MSG_ERROR(libmsql.a not found in /usr/local/Hughes/lib; you MUST specify directory where libmsql.a resides using --with-dblibdir=)
	fi
else
	if test -f "$with_dblibdir/libmsql.a"; then
		mh_db_lib_dir="${with_dblibdir}"
	else
		AC_MSG_ERROR(libmsql.a not found in $with_dblibdir)
	fi
fi
DB_LIBS="-L$mh_db_lib_dir -lmsql"
AC_MSG_RESULT(found in $mh_db_lib_dir)
])dnl

dnl ---------------------------------------------------------------------------
dnl Setup all mySQL stuff
dnl ---------------------------------------------------------------------------
AC_DEFUN([MH_CHECK_MYSQL],
[
DBINC="mysql.h"
DBLIB1=""
DBLIB2=""
DBLIB3=""
DBLIB4=""
mh_db_inc_dir=""
mh_db_lib_dir=""
mh_db_name="MySQL"
AC_MSG_CHECKING(location of mySQL header file)
if test "$with_dbincdir" = no; then
	mysql_inc_dirs="\
	    ${HOME}/include           \
	    /usr/include/mysql        \
	    /usr/local/include        \
	    /usr/contrib/include      \
	    /opt/include              \
	    /usr/include              \
	    /usr/unsupported/include"
	for ac_dir in $mysql_inc_dirs ; do
	  if test -f $ac_dir/mysql.h; then
	    mh_db_inc_dir=$ac_dir
	    break
	  fi
	done
	if test "x$mh_db_inc_dir" = "x"; then
		AC_MSG_ERROR(mysql.h not found; you MUST specify directory where mysql.h resides using --with-dbincdir=)
	fi
else
	if test -f "$with_dbincdir/mysql.h"; then
		mh_db_inc_dir="${with_dbincdir}"
	else
		AC_MSG_ERROR(mysql.h not found in $with_dbincdir)
	fi
fi
DB_INCLUDES="-I$mh_db_inc_dir"
AC_MSG_RESULT(found in $mh_db_inc_dir)

AC_MSG_CHECKING(location of mySQL library file)
if test "$with_dblibdir" = no; then
	mysql_lib_dirs="\
	    ${HOME}/lib               \
	    /usr/lib/mysql            \
	    /usr/local/lib            \
	    /usr/contrib/lib          \
	    /opt/lib                  \
	    /opt/sfw/lib              \
	    /usr/lib                  \
	    /usr/unsupported/lib"
	for ac_dir in $mysql_lib_dirs ; do
		for mh_ext in a so sl lib dylib; do
		  if test -r $ac_dir/libmysqlclient.$mh_ext; then
		     mh_db_lib_dir=$ac_dir
		     break 2
		  fi
		done
	done

	if test "x$mh_db_lib_dir" = "x"; then
		AC_MSG_ERROR(libmysqlclient.a not found; you MUST specify directory where libmysqlclient.a resides using --with-dblibdir=)
	fi
else
	if test -f "$with_dblibdir/libmysqlclient.a"; then
		mh_db_lib_dir="${with_dblibdir}"
	else
		AC_MSG_ERROR(libmysqlclient.a not found in $with_dblibdir)
	fi
fi

DB_LIBS="-L$mh_db_lib_dir -lmysqlclient"
case "$target" in
	*linux*)
	DB_LIBS="$DB_LIBS -lz"
	;;
esac
AC_MSG_RESULT(found in $mh_db_lib_dir)
])dnl

dnl ---------------------------------------------------------------------------
dnl Setup all PostgreSQL ODBC stuff
dnl ---------------------------------------------------------------------------
AC_DEFUN([MH_CHECK_PGSQLODBC],
[
DBINC="isql.h"
DBLIB1=""
DBLIB2=""
DBLIB3=""
DBLIB4=""
mh_db_inc_dir=""
mh_db_lib_dir=""
mh_db_name="PostgreSQL ODBC"
AC_MSG_CHECKING(location of PostgreSQL ODBC header and library files)
if test "$with_dbincdir" = no; then
	AC_MSG_ERROR(you MUST specify directory where isql.h resides using --with-dbincdir=)
else
	if test -f "$with_dbincdir/isql.h"; then
		DB_INCLUDES="-I${with_dbincdir}"
		mh_db_inc_dir="$with_dbincdir"
	else
		AC_MSG_ERROR(isql.h not found in $with_dbincdir)
	fi
fi
if test "$with_dblibdir" = no; then
	AC_MSG_ERROR(you MUST specify directory where libpsqlodbc.${SHL} or libpsqlodbc.a resides using --with-dblibdir=)
else
	if test -f "$with_dblibdir/libpsqlodbc.${SHL}" -o -f "$with_dblibdir/libpsqlodbc.a"; then
		DB_LIBS="-L${with_dblibdir} -lpsqlodbc"
		mh_db_lib_dir="$with_dblibdir"
	else
		AC_MSG_ERROR(libpsqlodbc.${SHL} not found in $with_dblibdir)
	fi
fi
AC_MSG_RESULT(found)
])dnl

dnl ---------------------------------------------------------------------------
dnl Setup all unixODBC ODBC stuff
dnl ---------------------------------------------------------------------------
AC_DEFUN([MH_CHECK_UNIXODBC],
[
DBINC="sqlext.h"
DBLIB1=""
DBLIB2=""
DBLIB3=""
DBLIB4=""
mh_db_inc_dir=""
mh_db_lib_dir=""
mh_db_name="unixODBC"
AC_MSG_CHECKING(location of unixODBC header and library files)
if test "$with_dbincdir" = no; then
	AC_MSG_ERROR(you MUST specify directory where sqlext.h resides using --with-dbincdir=)
else
	if test -f "$with_dbincdir/sqlext.h"; then
		DB_INCLUDES="-I${with_dbincdir}"
		mh_db_inc_dir="$with_dbincdir"
	else
		AC_MSG_ERROR(sqlext.h not found in $with_dbincdir)
	fi
fi

if test "$with_dblibdir" = no; then
	AC_MSG_ERROR(you MUST specify directory where libodbc.${SHL} or libodbc.a resides using --with-dblibdir=)
else
	if test -f "$with_dblibdir/libodbc.${SHL}" -o -f "$with_dblibdir/libodbc.a"; then
		DB_LIBS="-L${with_dblibdir} -lodbc"
		mh_db_lib_dir="$with_dblibdir"
	else
		AC_MSG_ERROR(libodbc.${SHL} not found in $with_dblibdir)
	fi
fi
AC_MSG_RESULT(found)
])dnl

dnl ---------------------------------------------------------------------------
dnl Setup all easysoft ODBC-ODBC bridge stuff
dnl ---------------------------------------------------------------------------
AC_DEFUN([MH_CHECK_EASYSOFT_OOB],
[
DBINC="odbc.h"
DBLIB1=""
DBLIB2=""
DBLIB3=""
DBLIB4=""
mh_db_inc_dir=""
mh_db_lib_dir=""
mh_db_name="EasySoft ODBC-ODBC bridge"
AC_MSG_CHECKING(location of easysoft ODBC-ODBC bridge header file)
if test "$with_dbincdir" = no; then
	if test -f "/usr/local/easysoft/oob/client/include/odbc.h"; then
		mh_db_inc_dir="/usr/local/easysoft/oob/client/include"
	else
		AC_MSG_ERROR(odbcl.h not found in /usr/local/easysoft/oob/client/include; you MUST specify directory where odbc.h resides using --with-dbincdir=)
	fi
else
	if test -f "$with_dbincdir/odbc.h"; then
		mh_db_inc_dir="${with_dbincdir}"
	else
		AC_MSG_ERROR(odbc.h not found in $with_dbincdir)
	fi
fi
DB_INCLUDES="-I$mh_db_inc_dir"
AC_MSG_RESULT(found in $mh_db_inc_dir)

AC_MSG_CHECKING(location of easysoft ODBC-ODBC bridge library file)
if test "$with_dblibdir" = no; then
	if test -f "/usr/local/easysoft/oob/client/lib/libesoobclient.${SHL}"; then
		mh_db_lib_dir="/usr/local/easysoft/oob/client/lib"
	else
		AC_MSG_ERROR(libesoobclient.${SHL} not found in /usr/local/easysoft/oob/client/lib; you MUST specify directory where libmysqlclient.a resides using --with-dblibdir=)
	fi
else
	if test -f "$with_dblibdir/libesoobclient.${SHL}"; then
		mh_db_lib_dir="${with_dblibdir}"
	else
		AC_MSG_ERROR(libesoobclient.${SHL} not found in $with_dblibdir)
	fi
fi
DB_LIBS="-L$mh_db_lib_dir -lesoobclient"
AC_MSG_RESULT(found in $mh_db_lib_dir)
])dnl

dnl ---------------------------------------------------------------------------
dnl Setup all Openlink ODBC stuff
dnl ---------------------------------------------------------------------------
AC_DEFUN([MH_CHECK_OPENLINK],
[
DBINC="isql.h"
DBLIB1=""
DBLIB2=""
DBLIB3=""
DBLIB4=""
mh_db_inc_dir=""
mh_db_lib_dir=""
mh_db_name="OpenLink ODBC"
AC_MSG_CHECKING(location of Openlink header and library files)
if test "$with_dbincdir" = no; then
	AC_MSG_ERROR(you MUST specify directory where isql.h resides using --with-dbincdir=)
else
	if test -f "$with_dbincdir/isql.h"; then
		DB_INCLUDES="-I${with_dbincdir}"
		mh_db_inc_dir="$with_dbincdir"
	else
		AC_MSG_ERROR(isql.h not found in $with_dbincdir)
	fi
fi

if test "$with_dblibdir" = no; then
	AC_MSG_ERROR(you MUST specify directory where libiodbc.${SHL} or libiodbc.a resides using --with-dblibdir=)
else
	if test -f "$with_dblibdir/libiodbc.${SHL}" -o -f "$with_dblibdir/libiodbc.a"; then
		DB_LIBS="-L${with_dblibdir} -liodbc"
		mh_db_lib_dir="$with_dblibdir"
	else
		AC_MSG_ERROR(libiodbc.${SHL} not found in $with_dblibdir)
	fi
fi
AC_MSG_RESULT(found)
])dnl

dnl ---------------------------------------------------------------------------
dnl Setup all Openlink UDBC stuff
dnl ---------------------------------------------------------------------------
AC_DEFUN([MH_CHECK_OPENLINK_UDBC],
[
DBINC="libudbc.h"
DBLIB1=""
DBLIB2=""
DBLIB3=""
DBLIB4=""
mh_db_inc_dir=""
mh_db_lib_dir=""
mh_db_name="OpenLink UDBC"
AC_MSG_CHECKING(location of Openlink UDBC header and library files)
if test "$with_dbincdir" = no; then
	AC_MSG_ERROR(you MUST specify directory where libudbc.h resides using --with-dbincdir=)
else
	if test -f "$with_dbincdir/libudbc.h"; then
		DB_INCLUDES="-I${with_dbincdir}"
		mh_db_inc_dir="$with_dbincdir"
	else
		AC_MSG_ERROR(libudbc.h not found in $with_dbincdir)
	fi
fi

if test "$with_dblibdir" = no; then
	AC_MSG_ERROR(you MUST specify directory where libudbc.${SHL} or libudbc.a resides using --with-dblibdir=)
else
	if test -f "$with_dblibdir/libudbc.${SHL}" -o -f "$with_dblibdir/libudbc.a"; then
		DB_LIBS="-L${with_dblibdir} -ludbc"
		mh_db_lib_dir="$with_dblibdir"
	else
		AC_MSG_ERROR(libudbc.${SHL} or libudbc.a not found in $with_dblibdir)
	fi
fi
AC_MSG_RESULT(found)
])dnl

dnl ---------------------------------------------------------------------------
dnl Setup all Solid ODBC stuff
dnl ---------------------------------------------------------------------------
AC_DEFUN([MH_CHECK_SOLID],
[
DBINC="cli0defs.h"
DBLIB1=""
DBLIB2=""
DBLIB3=""
DBLIB4=""
mh_db_inc_dir=""
mh_db_lib_dir=""
mh_db_name="Solid Server"
AC_MSG_CHECKING(location of Solid header and library files)
if test "$with_dbincdir" = no; then
	AC_MSG_ERROR(you MUST specify the directory where cli0defs.h resides using --with-dbincdir=)
else
	if test -f "$with_dbincdir/cli0defs.h"; then
		DB_INCLUDES="-I${with_dbincdir}"
		mh_db_inc_dir="$with_dbincdir"
	else
		AC_MSG_ERROR(cli0defs.h not found in $with_dbincdir)
	fi
fi

if test "$with_dblibdir" = no; then
	AC_MSG_ERROR(you MUST specify the directory where scl${solid_os}nn.a {nn is version number} resides using --with-dblibdir=)
else
	if test -f "$with_dblibdir/libsolcli.${SHL}" -o -f "$with_dblibdir/libsolcli.a"; then
		DB_LIBS="-L${with_dblibdir} -lsolcli"
		mh_db_lib_dir="$with_dblibdir"
	else
		AC_MSG_ERROR(libsolcli.${SHL} or libsolcli.a not found in $with_dblibdir)
	fi
fi
AC_MSG_RESULT(found)
])dnl

dnl ---------------------------------------------------------------------------
dnl Setup all Velocis ODBC stuff
dnl ---------------------------------------------------------------------------
AC_DEFUN([MH_CHECK_VELOCIS],
[
DBINC="sqlext.h"
DBLIB1=""
DBLIB2=""
DBLIB3=""
DBLIB4=""
mh_db_inc_dir=""
mh_db_lib_dir=""
mh_db_name="Raima Velocis"
if test -n "$RDSHOME" ; then
	if test "$with_dbincdir" = no; then
		DB_INCLUDES="-I${RDSHOME}/include"
		mh_db_inc_dir="$RDSHOME/include"
	else
		DB_INCLUDES="-I${with_dbincdir}"
		mh_db_inc_dir="$with_dbincdir"
	fi

	if test "$with_dblibdir" = no; then
		DB_LIBS="-L${RDSHOME}/lib -lCadm -lCdict -lCenc -lCncp -lCrdbc -lCrdm -lCrm -lCrpc -lCuapi -lCsql"
		mh_db_lib_dir="$RDSHOME/lib"
	else
		DB_LIBS="-L${with_dblibdir} -lCadm -lCdict -lCenc -lCncp -lCrdbc -lCrdm -lCrm -lCrpc -lCuapi -lCsql"
		mh_db_lib_dir="$with_dblibdir"
	fi
else
	AC_MSG_ERROR(RDSHOME not set, cannot continue)
fi
])dnl

dnl ---------------------------------------------------------------------------
dnl Setup all SqlAnywhere stuff
dnl ---------------------------------------------------------------------------
AC_DEFUN([MH_CHECK_SQLANYWHERE],
[
DBINC="unixodbc.h"
DBLIB1=""
DBLIB2=""
DBLIB3=""
DBLIB4=""
mh_db_inc_dir=""
mh_db_lib_dir=""
mh_db_name="Sybase SQL Anywhere"
if test -n "$SQLANY" ; then
	if test "$with_dbincdir" = no; then
		DB_INCLUDES="-I${SQLANY}/include"
		mh_db_inc_dir="$SQLANY/include"
	else
		DB_INCLUDES="-I${with_dbincdir}"
		mh_db_inc_dir="$with_dbincdir"
	fi
	if test "$with_dblibdir" = no; then
		DBLIB1="${SQLANY}/lib/saodbc.so"
		mh_db_lib_dir="$SQLANY/lib"
	else
		DBLIB1="${with_dblibdir}/saodbc.so"
		mh_db_lib_dir="$with_dblibdir"
	fi
else
	AC_MSG_ERROR(SQLANY not set, cannot continue)
fi
])dnl

dnl ---------------------------------------------------------------------------
dnl Setup all DB2 stuff
dnl ---------------------------------------------------------------------------
AC_DEFUN([MH_CHECK_DB2],
[
DBINC="sqlcli.h"
DBLIB1=""
DBLIB2=""
DBLIB3=""
DBLIB4=""
mh_db_inc_dir=""
mh_db_lib_dir=""
mh_db_name="IBM DB2"
AC_MSG_CHECKING(location of DB2 header and library files)
if test "$with_dbincdir" = no; then
	AC_MSG_ERROR(you MUST specify the directory where sqlcli.h resides using --with-dbincdir=)
else
	if test -f "$with_dbincdir/sqlcli.h"; then
		DB_INCLUDES="-I${with_dbincdir}"
		mh_db_inc_dir="$with_dbincdir"
	else
	AC_MSG_ERROR(sqlcli.h not found in $with_dbincdir)
	fi
fi

if test "$with_dblibdir" = no; then
	AC_MSG_ERROR(you MUST specify directory where libdb2.${SHL} or libdb2.a resides using --with-dblibdir=)
else
	if test -f "$with_dblibdir/libdb2.${SHL}" -o -f "$with_dblibdir/libdb2.a"; then
	DB_LIBS="-L${with_dblibdir} -ldb2"
		mh_db_lib_dir="$with_dblibdir"
	else
	AC_MSG_ERROR(libdb2.${SHL} or libdb2.a not found in $with_dblibdir)
	fi
fi
AC_MSG_RESULT(found)
])dnl

dnl ---------------------------------------------------------------------------
dnl Setup all SQLite stuff
dnl ---------------------------------------------------------------------------
AC_DEFUN([MH_CHECK_SQLITE],
[
DBINC="sqlite.h"
DBLIB1=""
DBLIB2=""
DBLIB3=""
DBLIB4=""
mh_db_inc_dir=""
mh_db_lib_dir=""
mh_db_name="SQLite"
AC_MSG_CHECKING(location of SQLite header and library files)
if test "$with_dbincdir" = no; then
	AC_MSG_ERROR(you MUST specify directory where sqlite.h resides using --with-dbincdir=)
else
	if test -f "$with_dbincdir/sqlite.h"; then
		DB_INCLUDES="-I${with_dbincdir}"
		mh_db_inc_dir="$with_dbincdir"
	else
		AC_MSG_ERROR(sqlite.h not found in $with_dbincdir)
	fi
fi

if test "$with_dblibdir" = no; then
	AC_MSG_ERROR(you MUST specify directory where libsqlite.${SHL} or libsqlite.a resides using --with-dblibdir=)
else
	if test -f "$with_dblibdir/libsqlite.${SHL}" -o -f "$with_dblibdir/libsqlite.a"; then
		DB_LIBS="-L${with_dblibdir} -lsqlite"
		mh_db_lib_dir="$with_dblibdir"
	else
		AC_MSG_ERROR(libsqlite.${SHL} not found in $with_dblibdir)
	fi
fi
AC_MSG_RESULT(found)
])dnl

dnl ---------------------------------------------------------------------------
dnl Setup all Makefile variables
dnl ---------------------------------------------------------------------------
AC_DEFUN([MH_SETUP_MAKEFILE],
[
case "$dbtouse" in
	oracle)
		DBH="ora"
		DBC="ora"
		LDB="ora"
		BDB="ORA"
		SDB=""
		dbsrc="ora"
		MH_CHECK_ORACLE
		;;
	pgsqlodbc)
		DBH="odbc"
		DBC="cli-odbc"
		LDB="pgodbc"
		BDB="PGODBC"
		SDB="-DPGSQLODBC"
		dbsrc="rexxcli"
		MH_CHECK_PGSQLODBC
		;;
	openlink)
		DBH="odbc"
		DBC="cli-odbc"
		LDB="odbc"
		BDB="ODBC"
		SDB="-DOPENLINK"
		dbsrc="rexxcli"
		MH_CHECK_OPENLINK
		;;
	udbc)
		DBH="udbc"
		DBC="cli-odbc"
		LDB="udbc"
		BDB="UDBC"
		SDB="-DUDBC"
		SDB=""
		dbsrc="rexxcli"
		MH_CHECK_OPENLINK_UDBC
		;;
	intersolv)
		DBH="odbc"
		DBC="cli-odbc"
		LDB="odbc"
		BDB="ODBC"
		SDB="-DINTERSOLV"
		dbsrc="rexxcli"
		;;
	sybase)
		DBH="syb"
		DBC="syb"
		LDB="syb"
		BDB="SYB"
		SDB=""
		dbsrc="syb"
		MH_CHECK_SYBASE
		;;
	sqlanywhere)
		DBH="saw"
		DBC="cli-odbc"
		LDB="saw"
		BDB="SAW"
		SDB=""
		dbsrc="rexxcli"
		MH_CHECK_SQLANYWHERE
		;;
	msql)
		DBH="min2"
		DBC="min2"
		LDB="min"
		BDB="MIN"
		SDB=""
		dbsrc="min"
		MH_CHECK_MSQL
		;;
	mysql)
		DBH="my"
		DBC="my"
		LDB="my"
		BDB="MY"
		SDB=""
		dbsrc="my"
		MH_CHECK_MYSQL
		;;
	db2)
		DBH="db2"
		DBC="cli-odbc"
		LDB="db2"
		BDB="DB2"
		SDB=""
		dbsrc="rexxcli"
		MH_CHECK_DB2
		;;
	solid)
		DBH="sol"
		DBC="cli-odbc"
		LDB="sol"
		BDB="SOL"
		SDB="-DSS_UNIX -DSOLID_SERVER"
		dbsrc="rexxcli"
		MH_CHECK_SOLID
		;;
	velocis)
		DBH="vel"
		DBC="cli-odbc"
		LDB="vel"
		BDB="VEL"
		SDB="-DVELOCIS"
		dbsrc="rexxcli"
		MH_CHECK_VELOCIS
		;;
	unixodbc)
		DBH="odbc"
		DBC="cli-odbc"
		LDB="odbc"
		BDB="ODBC"
		SDB="-DUNIXODBC"
		dbsrc="rexxcli"
		MH_CHECK_UNIXODBC
		;;
	easysoft-oob)
		DBH="esoob"
		DBC="cli-odbc"
		LDB="esoob"
		BDB="ESOOB"
		SDB="-DEASYSOFT_OOB"
		dbsrc="rexxcli"
		MH_CHECK_EASYSOFT_OOB
		;;
	sqlite)
		DBH="sqlite"
		DBC="sqlite"
		LDB="lite"
		BDB="LITE"
		SDB="-DSQLITE"
		dbsrc="sqlite"
		MH_CHECK_SQLITE
		;;
	none)
		;;
	*)
		AC_MSG_ERROR("No database specified to be configured")
		;;
esac
AC_SUBST(DBH)
AC_SUBST(DEFS)
AC_SUBST(DBC)
AC_SUBST(LDB)
AC_SUBST(BDB)
AC_SUBST(SDB)
AC_SUBST(dbsrc)
AC_SUBST(DB_LIBS)
AC_SUBST(DB_INCLUDES)
AC_SUBST(DBLIB1)
AC_SUBST(DBLIB2)
AC_SUBST(DBLIB3)
AC_SUBST(DBLIB4)
AC_SUBST(INCLUDE_DB_MAKEFILE)
])dnl

dnl
dnl include our own common routines
dnl
sinclude(common/accommon.m4)dnl
