/*
 * rxsqlinc.h - common routines for Rexx/SQL
 *
 *
 * REXX/SQL. A Rexx interface to SQL databases.
 * Copyright Mark Hessling, 1996-2002.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to:
 *
 *    The Free Software Foundation, Inc.
 *    675 Mass Ave,
 *    Cambridge, MA 02139 USA.
 *
 *
 * If you make modifications to this software that you feel increases
 * it usefulness for the rest of the community, please email the
 * changes, enhancements, bug fixes as well as any and all ideas to 
 * address below.
 * This software is going to be maintained and enhanced as deemed
 * necessary by the community.
 *
 * Mark Hessling  M.Hessling@qut.edu.au  http://www.lightlink.com/hessling/
 *
 */

/*-----------------------------------------------------------------------------
 * SYNOPSIS:  SQLCOMMIT( )
 *
 * RETURNS :  0-success, <0-error.
 *----------------------------------------------------------------------------*/
RFH_RETURN_TYPE SQLCOMMIT

#ifdef HAVE_PROTO
    ( RFH_ARG0_TYPE name, RFH_ARG1_TYPE argc, RFH_ARG2_TYPE argv, RFH_ARG3_TYPE stck, RFH_ARG4_TYPE retstr )
#else
    ( name, argc, argv, stck, retstr)
    RFH_ARG0_TYPE name;
    RFH_ARG1_TYPE argc;
    RFH_ARG2_TYPE argv;
    RFH_ARG3_TYPE stck;
    RFH_ARG4_TYPE retstr;
#endif
{ 
   return(SQLTRANSACT(name, argc, argv, stck, retstr,SQL_COMMIT));
}

/*-----------------------------------------------------------------------------
 * SYNOPSIS:  SQLROLLBACK( )
 *
 * RETURNS :  0-success, <0-error.
 *----------------------------------------------------------------------------*/
RFH_RETURN_TYPE SQLROLLBACK

#ifdef HAVE_PROTO
    ( RFH_ARG0_TYPE name, RFH_ARG1_TYPE argc, RFH_ARG2_TYPE argv, RFH_ARG3_TYPE stck, RFH_ARG4_TYPE retstr )
#else
    ( name, argc, argv, stck, retstr)
    RFH_ARG0_TYPE name;
    RFH_ARG1_TYPE argc;
    RFH_ARG2_TYPE argv;
    RFH_ARG3_TYPE stck;
    RFH_ARG4_TYPE retstr;
#endif
{ 
   return(SQLTRANSACT(name, argc, argv, stck, retstr,SQL_ROLLBACK));
}

/*-----------------------------------------------------------------------------
 * SYNOPSIS:  SQLOPEN(statement-name [, <bind-args>])
 *
 * RETURNS :  0-success,
 *           >0-number of rows affected for SQLEXECUTE(),
 *           <0-error.
 *----------------------------------------------------------------------------*/
RFH_RETURN_TYPE SQLOPEN

#ifdef HAVE_PROTO
    ( RFH_ARG0_TYPE name, RFH_ARG1_TYPE argc, RFH_ARG2_TYPE argv, RFH_ARG3_TYPE stck, RFH_ARG4_TYPE retstr )
#else
    ( name, argc, argv, stck, retstr)
    RFH_ARG0_TYPE name;
    RFH_ARG1_TYPE argc;
    RFH_ARG2_TYPE argv;
    RFH_ARG3_TYPE stck;
    RFH_ARG4_TYPE retstr;
#endif
{
   return(SQLEXEC(name, argc, argv, stck, retstr,1));
}

/*-----------------------------------------------------------------------------
 * SYNOPSIS:  SQLEXECUTE(statement-name [, <bind-args>])
 *
 * RETURNS :  0-success,
 *           >0-number of rows affected for SQLEXECUTE(),
 *           <0-error.
 *----------------------------------------------------------------------------*/
RFH_RETURN_TYPE SQLEXECUTE

#ifdef HAVE_PROTO
    ( RFH_ARG0_TYPE name, RFH_ARG1_TYPE argc, RFH_ARG2_TYPE argv, RFH_ARG3_TYPE stck, RFH_ARG4_TYPE retstr )
#else
    ( name, argc, argv, stck, retstr)
    RFH_ARG0_TYPE name;
    RFH_ARG1_TYPE argc;
    RFH_ARG2_TYPE argv;
    RFH_ARG3_TYPE stck;
    RFH_ARG4_TYPE retstr;
#endif
{
   return(SQLEXEC(name, argc, argv, stck, retstr,0));
}

/*-----------------------------------------------------------------------------
 * SYNOPSIS:  SQLVARIABLE( variable_name [,variable_value] )
 *
 * ARGUMENTS: 
 *            0 - variable_name
 *            1 - variable_value (if supplied, this is the new value)
 *
 * RETURNS :  Current value of "variable_name" or <0-error.
 *----------------------------------------------------------------------------*/
RFH_RETURN_TYPE SQLVARIABLE

#ifdef HAVE_PROTO
    ( RFH_ARG0_TYPE name, RFH_ARG1_TYPE argc, RFH_ARG2_TYPE argv, RFH_ARG3_TYPE stck, RFH_ARG4_TYPE retstr )
#else
    ( name, argc, argv, stck, retstr)
    RFH_ARG0_TYPE name;
    RFH_ARG1_TYPE argc;
    RFH_ARG2_TYPE argv;
    RFH_ARG3_TYPE stck;
    RFH_ARG4_TYPE retstr;
#endif
{
   char        opt_name[MAX_IDENTIFIER+1];
   SQLVAL      *opt=NULL;
   char        tmp[128];
   int         rc=0;
   RXSTRING    *rxs=NULL;
   ULONG       new_num=0L;

   RexxSQLData.RxPackageGlobalData = FunctionPrologue( RexxSQLData.RxPackageGlobalData, RexxSQLInitialiser, (char*)name, argc, argv);
   
   if (RexxSQLData.SQLCA_SqlCode) ClearDBError();
   if (RexxSQLData.SQLCA_IntCode) ClearIntError();
   
   if (argc == 0 || argc > 2) return 1;
   
   if (RXSTRLEN(argv[0]) == 0)
      return ReturnError(retstr, 19, "null (\"\") variable name.");
   
   /* 
    * Get the name of the variable 
    */
   if ( (rc = MkIdentifier(argv[0], opt_name, sizeof(opt_name))) != 0 )
      return RxReturnNumber( RexxSQLData.RxPackageGlobalData, retstr, (long)rc);
   
   /* 
    * Make sure the option is a valid one! 
    */
   if ( ( opt = ll_find( RexxSQLData.opt_tbl, opt_name ) ) == (SQLVAL*)NULL )
   {
      (void)sprintf(tmp,"unknown variable \"%s\".", opt_name);
      return ReturnError(retstr, 11, tmp);
   }
   
   /*
    * If the option is to set debugging level, we need to do more than
    * set a variable!
    */
   if (opt->valtype == SQLVAL_DEBUG)
   {
      if (argc == 1)
         return RxReturnNumber( RexxSQLData.RxPackageGlobalData,  retstr, RxGetRunFlags( RexxSQLData.RxPackageGlobalData ) );
      else
      {
         if (StrToInt(&argv[1], &new_num))
            return ReturnError(retstr, 7,"value is not a valid integer.");
         RxSetRunFlags( RexxSQLData.RxPackageGlobalData, new_num );
         return RxReturnNumber( RexxSQLData.RxPackageGlobalData, retstr, 0 );
      }
   }
   
   /*
    * If the option is to set debug trace file, we need to do more than
    * set a variable!
    */
   if (opt->valtype == SQLVAL_DEBUGFILE)
   {
      if (argc == 1)
         return RxReturnString( RexxSQLData.RxPackageGlobalData, retstr, RxGetTraceFile( RexxSQLData.RxPackageGlobalData ) );
      else
      {
         if (RXSTRLEN(argv[1]) > MAX_PATH)
           {
            sprintf(tmp,"identifier is too long; max length is %d",MAX_IDENTIFIER);
            return ReturnError(retstr, 6,tmp);
           }
         rc = RxSetTraceFile( RexxSQLData.RxPackageGlobalData, RXSTRPTR(argv[1]) );
         return RxReturnNumber( RexxSQLData.RxPackageGlobalData, retstr, rc);
      }
   }
   
   if (argc == 1) 
   {
      if (opt->dtype == TYPE_INT
      ||  opt->dtype == TYPE_BOOL)
         return RxReturnNumber( RexxSQLData.RxPackageGlobalData, retstr, *(ULONG*)(opt->value));
      else
         return RxReturnString( RexxSQLData.RxPackageGlobalData, retstr, ((RXSTRING*)(opt->value))->strptr);
   }
   
   /* 
    * Must be updateable by user 
    */
   if (!opt->user_update) 
   {
      (void)sprintf(tmp, "variable \"%s\" is not settable.", opt_name);
      return ReturnError(retstr, 12, tmp);
   }
   
   /* 
    * Set the value of the option 
    */
   switch(opt->dtype)
   {
      case TYPE_INT:
         if (StrToInt(&argv[1], &new_num))
            return ReturnError(retstr, 7,"value is not a valid integer.");
         memcpy((ULONG*)opt->value,&new_num,sizeof(ULONG));
         break;
      case TYPE_BOOL:
         if (StrToBool(&argv[1], &new_num))
            return ReturnError(retstr, 97,"value is not a valid boolean.");
         memcpy((ULONG*)opt->value,&new_num,sizeof(ULONG));
         break;
      case TYPE_STRING:
         if (RXSTRLEN(argv[1]) > MAX_IDENTIFIER)
         {
            sprintf(tmp,"identifier is too long; max length is %d",MAX_IDENTIFIER);
            return ReturnError(retstr, 6,tmp);
         }
         rxs = (RXSTRING*)opt->value;
         RXSTRCPY(rxs->strptr,rxs->strlength,RXSTRPTR(argv[1]),RXSTRLEN(argv[1]));
         break;
      case TYPE_ARRAY:
         break;
   }
   
   return RxReturnNumber( RexxSQLData.RxPackageGlobalData, retstr, 0 );
}

/*-----------------------------------------------------------------------------
 * SYNOPSIS:  SQLGETINFO( [connection_name],  variable_name [, stem_name)
 *
 * ARGUMENTS: 
 *            0 - connection_name (optional)
 *            1 - variable_name
 *            2 - stem_name (optional)
 *
 * RETURNS :  0-success, <0-error if argument 3 supplied
 *            or Current value of "variable_name" if argument 3 NOT supplied
 *----------------------------------------------------------------------------*/
RFH_RETURN_TYPE SQLGETINFO

#ifdef HAVE_PROTO
    ( RFH_ARG0_TYPE name, RFH_ARG1_TYPE argc, RFH_ARG2_TYPE argv, RFH_ARG3_TYPE stck, RFH_ARG4_TYPE retstr )
#else
    ( name, argc, argv, stck, retstr)
    RFH_ARG0_TYPE name;
    RFH_ARG1_TYPE argc;
    RFH_ARG2_TYPE argv;
    RFH_ARG3_TYPE stck;
    RFH_ARG4_TYPE retstr;
#endif
{
   char     opt_name[MAX_IDENTIFIER+1];
   SQLVAL   *opt=NULL;
   char     tmp[128];
   int      rc=0,num_vars=0,i=0,len=0;
   RXSTRING *rxs=NULL;
   RXSTRING_DT *rxs_dt=NULL;
   DBCON    *db=NULL;
   char     varname[MAX_IDENTIFIER+1+10];
   char     dbname[MAX_IDENTIFIER+1];
   char     stem_name[MAX_IDENTIFIER+1];
  
   RexxSQLData.RxPackageGlobalData = FunctionPrologue( RexxSQLData.RxPackageGlobalData, RexxSQLInitialiser, (char*)name, argc, argv);
  
   if (RexxSQLData.SQLCA_SqlCode) ClearDBError();
   if (RexxSQLData.SQLCA_IntCode) ClearIntError();
  
   if (argc > 3 || argc < 2) return 1;
  
   if ( (rc = MkIdentifier(argv[1], opt_name, sizeof(opt_name))) != 0 )
      return RxReturnNumber( RexxSQLData.RxPackageGlobalData, retstr, (long)rc);
   if (argc == 3)
   {
      if ( (rc = MkIdentifier(argv[2], stem_name, sizeof(stem_name))) != 0 )
         return RxReturnNumber( RexxSQLData.RxPackageGlobalData, retstr, rc);
      /*
       * If the stem name does NOT contain a trailing '.', error...
       */
      if (stem_name[strlen(stem_name)-1] != '.')
         return ReturnError(retstr, 91, "stem name MUST have trailing '.'");
   }
   /*
    * Find the variable in the connection table...
    */
   if (RexxSQLData.DbEnv == (DBENV*)NULL)
   {
      return ReturnError(retstr, 22, "no connections open.");
   }
   if (RXSTRLEN(argv[0])) 
   {
      /*
       * We have a named connection, get it...
       */
      if ( (rc = MkIdentifier(argv[0], dbname, sizeof(dbname))) != 0 )
         return RxReturnNumber( RexxSQLData.RxPackageGlobalData, retstr, rc);
      /* 
       * Make sure the named connection exists. 
       */
      if ((db = FindObject(dbname, RexxSQLData.DbEnv->db_tbl, TBL_CONNECTIONS)) == (DBCON*)NULL)
      {
         (void)sprintf(tmp,"connection \"%s\" is not open.", dbname);
         return ReturnError(retstr, 21, tmp);
      }
   }
   else
   {
      if (RexxSQLData.DbEnv->current_connection)
         db = RexxSQLData.DbEnv->current_connection;
      else
         return ReturnError(retstr, 25, "no connection is current");
      strcpy(dbname,db->name);
   }
   /* 
    * Make sure the option is a valid one! 
    */
   if ((opt = ll_find(db->db_opt, opt_name)) == (SQLVAL*)NULL)
   {
      (void)sprintf(tmp,"unknown variable \"%s\".", opt_name);
      return ReturnError(retstr, 11, tmp);
   }
   /*
    * If caller requires return as a string, get the variable's values
    * and return it.
    */
   if (argc == 2)
   {
      switch(opt->dtype)
      {
         case TYPE_INT:
         case TYPE_BOOL:
            return RxReturnNumber( RexxSQLData.RxPackageGlobalData, retstr, *(ULONG*)(opt->value));
            break;
         case TYPE_STRING:
            return RxReturnString( RexxSQLData.RxPackageGlobalData, retstr,((RXSTRING*)(opt->value))->strptr);
            break;
         case TYPE_ARRAY:
            rxs =(RXSTRING*)(opt->value);
            for (len=0,i=0;rxs[i].strlength;i++,len+=(rxs[i].strlength)+1);
            if (len > 256)
            {
               if ((retstr->strptr = (RXSTRING_STRPTR_TYPE)malloc(len)) == NULL)
                  return ReturnError(retstr, 10, "out of memory");
            }
            len = 0;
            for (i=0;rxs[i].strlength;i++)
            {
               memcpy(retstr->strptr+len,rxs[i].strptr,rxs[i].strlength);
               len += rxs[i].strlength;
               memcpy(retstr->strptr+len," ",1);
               len++;
            }
            len--;
            retstr->strptr[len] = '\0';
            retstr->strlength = len;
            return (0);
            break;
         case TYPE_DATATYPE:
            len = 0;
            rxs_dt =(RXSTRING_DT*)(opt->value);
            if (rxs_dt != NULL)
            {
               for (len=0,i=0;rxs_dt[i].strlength;i++,len+=(rxs_dt[i].strlength)+1);
               if (len > 256)
               {
                  if ((retstr->strptr = (RXSTRING_STRPTR_TYPE)malloc(len)) == NULL)
                     return ReturnError(retstr, 10, "out of memory");
               }
               len = 0;
               for (i=0;rxs_dt[i].strlength;i++)
               {
                  memcpy(retstr->strptr+len,rxs_dt[i].strptr,rxs_dt[i].strlength);
                  len += rxs_dt[i].strlength;
                  memcpy(retstr->strptr+len," ",1);
                  len++;
               }
               len--;
            }
            retstr->strptr[len] = '\0';
            retstr->strlength = len;
            return (0);
            break;
      }
   }
   /*
    * To reach here the caller supplied a stem name. Use it to generate
    * a REXX array.
    */
   switch(opt->dtype)
   {
      case TYPE_INT:
      case TYPE_BOOL:
         (void)sprintf(varname, "%s%u", stem_name, 1);
         sprintf(tmp,"%lu",*(ULONG*)(opt->value));
         num_vars = 1;
         rc = SetRexxVariable( RexxSQLData.RxPackageGlobalData, varname, strlen(varname), tmp, strlen(tmp));
         break;
      case TYPE_STRING:
         (void)sprintf(varname, "%s%u", stem_name, 1);
         num_vars = 1;
         rc = SetRexxVariable( RexxSQLData.RxPackageGlobalData, varname, strlen(varname),
                       ((RXSTRING*)(opt->value))->strptr,
                       (size_t)(((RXSTRING*)(opt->value))->strlength));
         break;
      case TYPE_ARRAY:
         rxs =(RXSTRING*)(opt->value);
         for (i=0;;i++)
         {
            if (rxs[i].strlength == 0)
               break;
            (void)sprintf(varname, "%s%u", stem_name, i+1);
            rc = SetRexxVariable( RexxSQLData.RxPackageGlobalData, varname, strlen(varname), rxs[i].strptr,rxs[i].strlength);
         }
         num_vars = i;
         break;
      case TYPE_DATATYPE:
         rxs_dt =(RXSTRING_DT*)(opt->value);
         if (rxs_dt)
         {
            for (i=0;;i++)
            {
               if (rxs_dt[i].strlength == 0)
                  break;
               (void)sprintf(varname, "%s%u", stem_name, i+1);
               rc = SetRexxVariable( RexxSQLData.RxPackageGlobalData, varname, strlen(varname), rxs_dt[i].strptr,rxs_dt[i].strlength);
            }
         }
         else
         {
            i = 1;
            (void)sprintf(varname, "%s%u", stem_name, i);
            rc = SetRexxVariable( RexxSQLData.RxPackageGlobalData, varname, strlen(varname), "*** not supported ***",21);
         }
         num_vars = i;
         break;
   }
   if (!rc)
   {
      (void)sprintf(varname, "%s0", stem_name);
      sprintf(tmp,"%u",num_vars);
      rc = SetRexxVariable( RexxSQLData.RxPackageGlobalData, varname, strlen(varname), tmp, strlen(tmp));
   }
  
   return RxReturnNumber( RexxSQLData.RxPackageGlobalData, retstr, rc);
}

/*-----------------------------------------------------------------------------
 * Declare the initiating and terminating functions when used as a dynamic lib
 *----------------------------------------------------------------------------*/

/*
 * SYNOPSIS: SQLLOADFUNCS();
 */
RFH_RETURN_TYPE SQLLOADFUNCS

#ifdef HAVE_PROTO
    ( RFH_ARG0_TYPE name, RFH_ARG1_TYPE argc, RFH_ARG2_TYPE argv, RFH_ARG3_TYPE stck, RFH_ARG4_TYPE retstr )
#else
    ( name, argc, argv, stck, retstr)
    RFH_ARG0_TYPE name;
    RFH_ARG1_TYPE argc;
    RFH_ARG2_TYPE argv;
    RFH_ARG3_TYPE stck;
    RFH_ARG4_TYPE retstr;
#endif
{
   int rc=0;

#if defined(DYNAMIC_LIBRARY)
   RexxSQLData.RxPackageGlobalData = InitRxPackage( RexxSQLData.RxPackageGlobalData, RexxSQLInitialiser, &rc );
   if ( !QueryRxFunction( RexxSQLData.RxPackageGlobalData, NAME_SQLCONNECT_EXT ) )
   {
      /*
       * Register all external functions
       */
      if ( !rc )
      {
         rc = RegisterRxFunctions( RexxSQLData.RxPackageGlobalData, RexxSQLFunctions, RxPackageName );
      }
   }
#endif
   return RxReturnNumber( RexxSQLData.RxPackageGlobalData,  retstr, rc );
}

/*
 * SYNOPSIS: SQLDROPFUNCS(["UNLOAD"]);
 */
RFH_RETURN_TYPE SQLDROPFUNCS

#ifdef HAVE_PROTO
    ( RFH_ARG0_TYPE name, RFH_ARG1_TYPE argc, RFH_ARG2_TYPE argv, RFH_ARG3_TYPE stck, RFH_ARG4_TYPE retstr )
#else
    ( name, argc, argv, stck, retstr)
    RFH_ARG0_TYPE name;
    RFH_ARG1_TYPE argc;
    RFH_ARG2_TYPE argv;
    RFH_ARG3_TYPE stck;
    RFH_ARG4_TYPE retstr;
#endif
{
   int rc=0;
   int unload=0;

   RexxSQLData.RxPackageGlobalData = FunctionPrologue( RexxSQLData.RxPackageGlobalData, RexxSQLInitialiser, (char*)name, argc, argv );

   if ( my_checkparam( RexxSQLData.RxPackageGlobalData, NAME_SQLDROPFUNCS_INT, argc, 0, 1 ) )
      return( 1 );
   if ( argv[0].strlength == 6
   &&   memcmpi( argv[0].strptr, "UNLOAD", 6 ) == 0 )
      unload = 1;
   (void)TermRxPackage( &RexxSQLData.RxPackageGlobalData, RexxSQLTerminator, RexxSQLFunctions, RxPackageName, unload );
   return RxReturnNumber( NULL, retstr, rc );
}

/*-----------------------------------------------------------------------------
 * Set up the Version String
 *----------------------------------------------------------------------------*/
static void *SetVersion

#if HAVE_PROTO
    (void *db)
#else
    (db)
    void *db;
#endif

{
   InternalTrace( RexxSQLData.RxPackageGlobalData, "SetVersion", "%x", db );
   sprintf(RexxSqlVersionString,"%s %s %s %s %s",
                              RxPackageName,
                              REXXSQL_VERSION,
                              REXXSQL_DATE,
                              CURRENT_OS,
                              CURRENT_DB);
   MAKERXSTRING(RexxSqlVersion, RexxSqlVersionString,(ULONG)strlen(RexxSqlVersionString));
   return((void*)&RexxSqlVersion);
}

/*-----------------------------------------------------------------------------
 * Set up the string representing a NULL value
 *----------------------------------------------------------------------------*/
static void *SetDebugFile

#if HAVE_PROTO
    (void *db)
#else
    (db)
    void *db;
#endif
{

   InternalTrace( RexxSQLData.RxPackageGlobalData, "SetDebugFile", "%x", db );

   strcpy( RexxSQLData.DebugFile, DEBUGFILE );
   MAKERXSTRING( RexxSQLData.RXSDebugFile, RexxSQLData.DebugFile, (ULONG)strlen( RexxSQLData.DebugFile ) );
   return( (void*)&RexxSQLData.RXSDebugFile );
}

/*-----------------------------------------------------------------------------
 * Set up the string representing a NULL value
 *----------------------------------------------------------------------------*/
static void *SetNullStringIn

#if HAVE_PROTO
    (void *db)
#else
    (db)
    void *db;
#endif
{

   InternalTrace( RexxSQLData.RxPackageGlobalData, "SetNullStringIn", "%x", db );

   strcpy( RexxSQLData.NullStringIn, NULLSTRINGIN );
   MAKERXSTRING( RexxSQLData.RXSNullStringIn, RexxSQLData.NullStringIn, (ULONG)strlen( RexxSQLData.NullStringIn ) );
   return( (void*)&RexxSQLData.RXSNullStringIn );
}

/*-----------------------------------------------------------------------------
 * Set up the string representing a NULL value
 *----------------------------------------------------------------------------*/
static void *SetNullStringOut

#if HAVE_PROTO
    (void *db)
#else
    (db)
    void *db;
#endif

{

   InternalTrace( RexxSQLData.RxPackageGlobalData, "SetNullStringOut", "%x", db );

   strcpy( RexxSQLData.NullStringOut, NULLSTRINGOUT );
   MAKERXSTRING( RexxSQLData.RXSNullStringOut, RexxSQLData.NullStringOut, (ULONG)strlen( RexxSQLData.NullStringOut ) );
   return( (void*)&RexxSQLData.RXSNullStringOut);
}

/*-----------------------------------------------------------------------------
 * Install all REXX/SQL variable structures.
 *----------------------------------------------------------------------------*/
void *GetSqlVariableValue

#ifdef HAVE_PROTO
    (int type, void *idb)
#else
    (type,idb)
    int type;
    void *idb;
#endif

{
   void *result=NULL;
   DBCON *db=idb;

   InternalTrace( RexxSQLData.RxPackageGlobalData, "GetSqlVariableValue", "%d,%x", type, idb );

   switch (type)
   {
      case SQLVAL_DEBUGFILE:
         result = SetDebugFile(NULL);
         break;
      case SQLVAL_NULLSTRINGIN:
         result = SetNullStringIn(NULL);
         break;
      case SQLVAL_NULLSTRINGOUT:
         result = SetNullStringOut(NULL);
         break;
      case SQLVAL_VERSION:
         result = SetVersion(NULL);
         break;
      case SQLVAL_SUPPORTSTRANSACTIONS:
         result = SetSupportsTransactions(db);
         break;
      case SQLVAL_SUPPORTSSQLGETDATA:
         result = SetSupportsSqlGetData(db);
         break;
      case SQLVAL_DESCRIBECOLUMNS:
         result = SetDescribeColumns(db);
         break;
      case SQLVAL_DATATYPES:
         result = SetDatatypes(db);
         break;
      case SQLVAL_DBMSNAME:
         result = SetDBMSName(db);
         break;
      case SQLVAL_DBMSVERSION:
         result = SetDBMSVer(db);
         break;
   }
   return (result);
}

/*-----------------------------------------------------------------------------
 * Install all REXX/SQL variable structures.
 *----------------------------------------------------------------------------*/
int InstallSQLVariables

#ifdef HAVE_PROTO
    (void)
#else
    ()
#endif

{
   int     rc=0,i=0;
   SQLVAL  *curr=NULL;
   void    *result=NULL;

   InternalTrace( RexxSQLData.RxPackageGlobalData, "InstallSQLVariables", NULL );

   for (i=0;strlen(sqlvariable_vars[i].name)!=0;i++)
   {
      if (sqlvariable_vars[i].value)
         result = sqlvariable_vars[i].value;
      else
         result = GetSqlVariableValue(sqlvariable_vars[i].valtype,NULL);
      if ((curr = InstallSQLVariable(
                &rc,
                sqlvariable_vars[i].name,
                RexxSQLData.opt_tbl,
                sqlvariable_vars[i].user_update, 
                sqlvariable_vars[i].dtype, 
                0, 
                sqlvariable_vars[i].option,
                result,
                sqlvariable_vars[i].valtype)) == NULL)
         return(rc);
      if ( RexxSQLData.opt_tbl == NULL )
         RexxSQLData.opt_tbl = curr;
   }
   return (0);
}

/*-----------------------------------------------------------------------------
 * Validates datatype string
 *----------------------------------------------------------------------------*/
static int ValidateDataType

#ifdef HAVE_PROTO
    (char *buf, int len, DBCON *db)
#else
    (buf, len, db)
    char          *buf;
    int           len;
    DBCON         *db;
#endif
{
   int         i=0;
   SQLVAL      *opt=NULL;
   RXSTRING_DT *rxs_dt=NULL;

   InternalTrace( RexxSQLData.RxPackageGlobalData, "ValidateDatatype", "%s,%d,%x", buf, len, db );

   /*
    * Find the internal datatype represenation for the supplied
    * datatype string. If not found, return -1.
    */
   if ((opt = ll_find(db->db_opt, "DATATYPES")) == (SQLVAL*)NULL)
      return (RXSQL_INVALID_DATATYPE);
   rxs_dt =(RXSTRING_DT*)(opt->value);
   for (i=0;rxs_dt[i].strlength;i++)
   {
    if (len == rxs_dt[i].strlength
    &&  memcmpi(rxs_dt[i].strptr,buf,len) == 0)
       return(rxs_dt[i].internal);
   }
   return(RXSQL_INVALID_DATATYPE);
}

/***********************************************************************/
RSH_RETURN_TYPE PackageSubcomHandler

#if defined(HAVE_PROTO)
   (
   RSH_ARG0_TYPE Command,    /* Command string passed from the caller    */
   RSH_ARG1_TYPE Flags,      /* pointer to short for return of flags     */
   RSH_ARG2_TYPE Retstr)     /* pointer to RXSTRING for RC return        */
#else
   ( Command, Flags, Retstr )
   RSH_ARG0_TYPE Command;    /* Command string passed from the caller    */
   RSH_ARG1_TYPE Flags;      /* pointer to short for return of flags     */
   RSH_ARG2_TYPE Retstr;     /* pointer to RXSTRING for RC return        */
#endif
/***********************************************************************/
{
   /*
    * Leave the Flags variable alone indicating we want to
    * use the default subcommand handler.
    */
   *Flags = *Flags;
   return 0;
}

/*
 * These functions used by loader.c to obtain package-specific info
 */
int RexxSQLInitialiser( RxPackageGlobalDataDef *MyGlob )
{
   int rc=0L;

   RexxSQLData.RxPackageGlobalData = MyGlob;
   RexxSQLData.RowLimit= 0L ;
   RexxSQLData.LongLimit = DEFAULT_LONG_LIMIT;
   RexxSQLData.SaveSQL = TRUE;
   RexxSQLData.AutoCommit = FALSE;
   RexxSQLData.IgnoreTruncate = FALSE;
   RexxSQLData.StandardPlacemarkers = FALSE;
   RexxSQLData.SupportsTransactions = SUPPORTSTRANSACTIONS;
   RexxSQLData.SupportsPlacemarkers = SUPPORTSPLACEMARKERS;
   RexxSQLData.SupportsDMLRowcount  = SUPPORTSDMLROWCOUNT;
   RexxSQLData.SupportsSqlGetData   = SUPPORTSSQLGETDATA;
   RexxSQLData.SQLCA_SqlCode = (-1L);
   RexxSQLData.SQLCA_RowCount = (-1L);
   RexxSQLData.SQLCA_IntCode = (-1L);
   RexxSQLData.column_attribute = col_attribute;
  
   InternalTrace( RexxSQLData.RxPackageGlobalData, "InitRexxSQL", NULL );
  
   /* 
    * Install REXX/SQL variable descriptors 
    */
   if ( (rc = InstallSQLVariables()) != 0 )
       return(rc);
   
   /*
    * Call any database-specific initialisation function here
    */
   if ( (rc = DBInitialise()) != 0 )
      return(rc);
  
   return 0;
}

int RexxSQLTerminator( RxPackageGlobalDataDef *MyGlob )
{
   int rc=0;
  
   RexxSQLData.RxPackageGlobalData = FunctionPrologue( RexxSQLData.RxPackageGlobalData, RexxSQLInitialiser, "TerminateRexxSQL", 0L, NULL);
  
   /* Release the REXX/SQL environment. */
   if ( (rc = ReleaseDbEnvironment()) != 0 )
      return (int)FunctionEpilogue( RexxSQLData.RxPackageGlobalData, "TerminateRexxSQL", (long)rc );
  
   RexxSQLData.opt_tbl = ll_free( RexxSQLData.opt_tbl );
  
   return (int)FunctionEpilogue( RexxSQLData.RxPackageGlobalData, "TerminateRexxSQL", (long)0 );
}

LONG APIENTRY RexxSQLInitHandler( LONG ExitNum, LONG Subfun, PEXIT PBlock )
{
/*   SetPackageConstants( rexxdwData.RxPackageGlobalData, RexxSQLConstants, "REXXSQL", 0 ); */
   return 0L;
}

RexxSubcomHandler *getRexxSQLSubcomHandler( void )
{
   return NULL;
}

RexxExitHandler *getRexxSQLInitHandler( void )
{
   return RexxSQLInitHandler;
}

RexxFunction *getRexxSQLFunctions( void )
{
   return RexxSQLFunctions;
}

RxPackageConstantDef *getRexxDQConstants( void )
{
   return NULL;
}

PackageInitialiser *getRexxSQLInitialiser( void )
{
   return RexxSQLInitialiser;
}

PackageTerminator *getRexxSQLTerminator( void )
{
   return RexxSQLTerminator;
}

#if !defined(HAVE_STRERROR)
/*
 * This function and the following description borrowed from Regina 0.08a
 *
 * Sigh! This must probably be done this way, although it's incredibly 
 * backwards. Some versions of gcc comes with a complete set of ANSI C
 * include files, which contains the definition of strerror(). However,
 * that function does not exist in the default libraries of SunOS. 
 * To circumvent that problem, strerror() is #define'd to get_sys_errlist() 
 * in config.h, and here follows the definition of that function. 
 * Originally, strerror() was #defined to sys_errlist[x], but that does
 * not work if string.h contains a declaration of the (non-existing) 
 * function strerror(). 
 *
 * So, this is a mismatch between the include files and the library, and
 * it should not create problems for Regina. However, the _user_ will not
 * encounter any problems until he compiles Regina, so we'll have to 
 * clean up after a buggy installation of the C compiler!
 */
char *rxsql_get_sys_errlist( int num )
{
   extern char *sys_errlist[] ;
   return sys_errlist[num] ;
}
#endif

/*-----------------------------------------------------------------------------
 * Parse the supplied statement and return 1 if the first word of the
 * statement is "select" (any case) or 0 otherwise.
 *----------------------------------------------------------------------------*/
static int SelectStatement

#ifdef HAVE_PROTO
    (char *stmt,long len)
#else
    (stmt,len)
    char    *stmt;
    long    len;
#endif

{
   char buf[7];
   char *p=stmt;
   int i=0;

   InternalTrace( RexxSQLData.RxPackageGlobalData, "SelectStatement", "%s,%d", stmt, len );
   /* Remove leading whitespace */
   while (len && isspace(*p)) 
   {
      p++;
      len--;
   }
   if (len<6L)
      return(0);
   buf[6] = '\0';
  
   for(i=0;i<6;i++)
   {
      buf[i] = (islower(*p)) ? toupper(*p) : *p;
      p++;
   }
   if (strcmp(buf,"SELECT") == 0)
      return(1);
   return(0);
}

/*-----------------------------------------------------------------------------
 * Store the error message for INTERNAL errors in SQLCA.SQLTEXT variable.
 *----------------------------------------------------------------------------*/
static int SetIntError

#ifdef HAVE_PROTO
    (char *fn, int lineno, int errcode, char *errmsg)
#else
    (fn,lineno,errcode,errmsg)
    char *fn;
    int   lineno;
    int   errcode;
    char *errmsg;
#endif

{
   char  msg[MAX_ERROR_TEXT];

   InternalTrace( RexxSQLData.RxPackageGlobalData, "SetIntError", "%s,%d,%d,%s", fn, lineno, errcode, errmsg );

   RexxSQLData.SQLCA_IntCode = -errcode;
   
   /* 
    * Set SQLCA.INTERRM variable
    */
   if ( RxGetRunFlags( RexxSQLData.RxPackageGlobalData ) == 10 )
      (void)sprintf(msg, "REXX/SQL-%02d: %s [%s:%d]", errcode, errmsg, fn, lineno);
   else
      (void)sprintf(msg, "REXX/SQL-%02d: %s", errcode, errmsg);
   (void)SetRexxVariable( RexxSQLData.RxPackageGlobalData, SQLCA_INTERRM, strlen(SQLCA_INTERRM),msg, strlen(msg));

   /* 
    * Set SQLCA.INTCODE variable 
    */
   (void)sprintf(msg, "%ld", RexxSQLData.SQLCA_IntCode);
   (void)SetRexxVariable( RexxSQLData.RxPackageGlobalData, SQLCA_INTCODE, strlen(SQLCA_INTCODE),msg, strlen(msg));

   return(RexxSQLData.SQLCA_IntCode);
}

/*-----------------------------------------------------------------------------
 * Set elements of the compound variable "SQLCA.".  SQLCA. holds the results of
 * any call to REXXSQL. It should never be set by the REXX program!
 *----------------------------------------------------------------------------*/
static void SetSQLCA

#ifdef HAVE_PROTO
    (RXSQL_SQLCODE_TYPE sqlcode, RXSQL_SQLSTATE_TYPE *sqlstate, RXSQL_SQLERRM_TYPE *errm, char *txt)
#else
    (sqlcode, sqlstate, errm, txt)
    RXSQL_SQLCODE_TYPE   sqlcode;
    RXSQL_SQLSTATE_TYPE  *sqlstate;
    RXSQL_SQLERRM_TYPE   *errm;
    char   *txt;
#endif

{
   char    buf[50];
  
   InternalTrace( RexxSQLData.RxPackageGlobalData, "SetSQLCA", "%d,%s,%s,%s", sqlcode, sqlstate, errm, txt );
  
   RexxSQLData.SQLCA_SqlCode = sqlcode;
   strcpy( RexxSQLData.SQLCA_SqlState, sqlstate );
  
   (void)sprintf(buf, "%ld", sqlcode);
   (void)SetRexxVariable( RexxSQLData.RxPackageGlobalData, SQLCA_SQLCODE,  strlen(SQLCA_SQLCODE),buf, strlen(buf));
   (void)SetRexxVariable( RexxSQLData.RxPackageGlobalData, SQLCA_SQLERRM,  strlen(SQLCA_SQLERRM),errm, strlen(errm));
   (void)SetRexxVariable( RexxSQLData.RxPackageGlobalData, SQLCA_SQLTEXT,  strlen(SQLCA_SQLTEXT),txt, strlen(txt));
   (void)SetRexxVariable( RexxSQLData.RxPackageGlobalData, SQLCA_SQLSTATE, strlen(SQLCA_SQLSTATE),sqlstate, strlen(sqlstate));
  
   if (!(sqlcode==0 && strcmp(sqlstate,"")==0 && strcmp(errm,"")==0))
   {
      (void)SetRexxVariable( RexxSQLData.RxPackageGlobalData, SQLCA_INTCODE, strlen(SQLCA_INTCODE),"-1", 2);
      (void)sprintf(buf, "REXX/SQL-1: Database Error");
      (void)SetRexxVariable( RexxSQLData.RxPackageGlobalData, SQLCA_INTERRM, strlen(SQLCA_INTERRM),buf, strlen(buf));
   }
   return;
}

/*-----------------------------------------------------------------------------
 * Set RowCount and put into REXX "SQLCA.ROWCOUNT" compound variable.
 *----------------------------------------------------------------------------*/
static void SetRowCount

#ifdef HAVE_PROTO
    (RXSQL_ROWCOUNT_TYPE rowcount)
#else
    (rowcount)
    RXSQL_ROWCOUNT_TYPE rowcount;
#endif

{
   char    buf[50];

   InternalTrace( RexxSQLData.RxPackageGlobalData, "SetRowCount", "%d", rowcount );

   if (RexxSQLData.SQLCA_RowCount != rowcount) 
   {
      RexxSQLData.SQLCA_RowCount = rowcount;
      (void)sprintf(buf, "%lu", rowcount);
      (void)SetRexxVariable( RexxSQLData.RxPackageGlobalData, SQLCA_ROWCOUNT,  strlen(SQLCA_ROWCOUNT),buf, strlen(buf));
   }
   return;
}

#if defined(USES_COMPUTE_COUNT)
/*-----------------------------------------------------------------------------
 * Set ComputeCount and put into REXX "SQLCA.COMPUTECOUNT" compound variable.
 *----------------------------------------------------------------------------*/
static void SetComputeCount

#ifdef HAVE_PROTO
    (RXSQL_COMPUTECOUNT_TYPE computecount)
#else
    (computecount)
    RXSQL_COMPUTECOUNT_TYPE computecount;
#endif

{
   char    buf[50];

   InternalTrace( RexxSQLData.RxPackageGlobalData, "SetComputeCount", "%d", computecount );

   if (RexxSQLData.SQLCA_ComputeCount != computecount) 
   {
      RexxSQLData.SQLCA_ComputeCount = computecount;
      (void)sprintf(buf, "%lu", computecount);
      (void)SetRexxVariable( RexxSQLData.RxPackageGlobalData, SQLCA_COMPUTECOUNT,  strlen(SQLCA_COMPUTECOUNT),buf, strlen(buf));
   }
   return;
}
#endif

/*-----------------------------------------------------------------------------
 * Clear the error code, error message etc.
 *----------------------------------------------------------------------------*/
static void ClearDBError

#ifdef HAVE_PROTO
    (void)
#else
    ()
#endif

{
#if defined(USES_MULTIPLE_ERRORS)
   char tmp[50];
#endif

   InternalTrace( RexxSQLData.RxPackageGlobalData, "ClearDBError", NULL );

   SetRowCount(0L);
#if defined(USES_COMPUTE_COUNT)
   SetComputeCount(0L);
#endif
#if defined(USES_MULTIPLE_ERRORS)
   sprintf(tmp,"%s.0",SQLCA_SQLERRM);
   (void)SetRexxVariable( RexxSQLData.RxPackageGlobalData, tmp, strlen(tmp), "0", 1);
   sprintf(tmp,"%s.0",SQLCA_SQLSTATE);
   (void)SetRexxVariable( RexxSQLData.RxPackageGlobalData, tmp, strlen(tmp), "0", 1);
#endif
   SetSQLCA(0L, "", "", "");
   return;
}

/*-----------------------------------------------------------------------------
 * Clear the internal error code, error message etc.
 *----------------------------------------------------------------------------*/
static void ClearIntError

#ifdef HAVE_PROTO
    (void)
#else
    ()
#endif

{
   RexxSQLData.SQLCA_IntCode = 0L;
  
   InternalTrace( RexxSQLData.RxPackageGlobalData, "ClearIntError", NULL );
  
   /* 
    * Set SQLCA.INTERRM variable 
    */
   (void)SetRexxVariable( RexxSQLData.RxPackageGlobalData, SQLCA_INTERRM, strlen(SQLCA_INTERRM),"", 0);
  
   /* 
    * Set SQLCA.INTCODE variable 
    */
   (void)SetRexxVariable( RexxSQLData.RxPackageGlobalData, SQLCA_INTCODE, strlen(SQLCA_INTCODE),"0", 1);
  
   return;
}

/*-----------------------------------------------------------------------------
 * Copy a string "s" to the RXSTRING structure pointed to by "target".
 * The string is not null terminated (unless the null is included as the
 * last byte of the string and is counted in "len".
 *----------------------------------------------------------------------------*/
static ULONG PutString

#ifdef HAVE_PROTO
    (RXSTRING *target, char *s, size_t len)
#else
    (target, s, len)
    RXSTRING   *target;
    char *s;
    size_t      len;
#endif

{
   char        *t=NULL;

   if (len > RXAUTOBUFLEN)
   {
#if defined(WIN32) && (defined(USE_OREXX) || defined(USE_WINREXX) || defined(USE_QUERCUS))
      if ((t = (char*)GlobalLock(GlobalAlloc(GMEM_FIXED,len+1))) == (char*)NULL)
#elif defined(USE_OS2REXX)
      if ((BOOL)DosAllocMem( (void**)&t, len+1, fPERM|PAG_COMMIT))
#else
      if ((t = (char*)malloc(len+1)) == (char*)NULL)
#endif
         return 1;
      target->strptr = (RXSTRING_STRPTR_TYPE)t;
   }

   memcpy(target->strptr, s, len);
   target->strlength = len;
   return 0;
}


/*-----------------------------------------------------------------------------
 * Return an error to REXX interpreter.
 *----------------------------------------------------------------------------*/
static ULONG ReturnError

#ifdef HAVE_PROTO
    (RXSTRING *retptr, int errcode, char *errmsg)
#else
    (retptr, errcode, errmsg)
    RXSTRING   *retptr;
    int         errcode;
    char *errmsg;
#endif

{
   SetIntError( __FILE__, __LINE__, errcode, errmsg);
   return RxReturnNumber( RexxSQLData.RxPackageGlobalData, retptr, -errcode);
}




/*-----------------------------------------------------------------------------
 * Makes an identifier (for a Connection, Stem or Statement) from a given
 * string. Leading and trailing whitespace is removed and the string is
 * capitalised. The variable must be the following regular expression:
 *   [A-Za-z!?][A-Za-z0-9!?_\.]*
 * That is the variable :-
 *   (i) The variable name must start with an alpha character or '!' or '?'.
 *  (ii) The rest of the variable name must be one of the above characters or
 *       a decimal digit or underscore or period.
 *----------------------------------------------------------------------------*/
static int MkIdentifier

#ifdef HAVE_PROTO
    (RXSTRING var, PSZ buf, size_t buflen)
#else
    (var, buf, buflen)
    RXSTRING  var;
    PSZ       buf;
    size_t    buflen;
#endif

{
   char      *p = RXSTRPTR(var);
   size_t    len = RXSTRLEN(var);
   size_t    cnt = 0;
  
   if ( len == 0 )
      return(SetIntError( __FILE__, __LINE__, 51, "zero length identifier"));
  
   /* 
    * Make room for the terminating byte. 
    */
   buflen--;
  
   /* 
    * Remove leading whitespace 
    */
   while( len && isspace(*p) )
   {
      p++;
      len--;
   }
  
#if 0
   /* 
    * Special check for 1st character 
    */
   if (len && (isalpha(*p) || *p == '!' || *p == '?')) 
   {
      *buf++ = (islower(*p)) ? toupper(*p) : *p;
      p++;
      len--;
   }
#endif
  
#if 0
   /* 
    * Copy identifier to destination 
    */
   while (len && (isalnum(*p) || *p == '_' || *p == '!' || *p == '?' || *p== '.'))
   {
      *buf = (islower(*p)) ? toupper(*p) : *p;
      p++;
      len--;
      if (++cnt <= buflen) 
         buf++;
   }
#else
   /* 
    * Copy identifier to destination 
    */
   while( len )
   {
      *buf = (islower(*p)) ? toupper(*p) : *p;
      p++;
      len--;
      if (++cnt <= buflen) 
         buf++;
   }
#endif
  
   /* 
    * Remove trailing whitespace 
    */
   while( len && isspace(*p) )
   {
      p++;
      len--;
   }
  
#if 0
   /* 
    * Check for garbage 
    */
   if (len)
      return(SetIntError( __FILE__, __LINE__, 52, "garbage in identifier name" ));
#endif
  
   *buf = '\0';
   return 0;
}

/*-----------------------------------------------------------------------------
 * Save SQL statement in cursor statement buffer.  Dynamically extends the
 * statement buffer when required.
 *----------------------------------------------------------------------------*/
static int SaveSqlStatement

#ifdef HAVE_PROTO
    (char **sql_stmt, int *sql_stmt_sz, char *statement, int length)
#else
    (sql_stmt, sql_stmt_sz, statement, length)
    char    **sql_stmt;
    int     *sql_stmt_sz;
    char    *statement;
    int     length;
#endif

{
   int len=length+1;
   char *ptr=NULL;
  
   InternalTrace( RexxSQLData.RxPackageGlobalData, "SaveSqlStatement", "%x,%x,%s,%d", sql_stmt, sql_stmt_sz, statement, length );
  
   if (len > *sql_stmt_sz)
   {
      /* New statement size is > than previous */
      if (*sql_stmt != (char*)NULL)
      {
         /* Free old statement */
         free(*sql_stmt); 
         *sql_stmt = (char*)NULL;
         *sql_stmt_sz = 0;
      }
  
      /* Allocate a buffer for the new statement */
      if ((*sql_stmt = (char*)malloc(len)) == (char*)NULL)
         return(SetIntError( __FILE__, __LINE__, 10, "out of memory"));
      *sql_stmt_sz = len;
   }
  
   /* Save the statement */
   (void)memcpy(*sql_stmt, statement, length);
   ptr=*sql_stmt;
   ptr[length] = '\0';
   /* set SQLCA.SQLTEXT */
   (void)SetRexxVariable( RexxSQLData.RxPackageGlobalData, SQLCA_SQLTEXT, strlen(SQLCA_SQLTEXT), statement, length );
   return 0;
}

/*-----------------------------------------------------------------------------
 * Count the number of '?' in a SQL statement. This is a kludge for ODBC
 * drivers that don't support SQLNumParams.
 *----------------------------------------------------------------------------*/
static int CountQuestionMarks

#ifdef HAVE_PROTO
    (char *sql_stmt)
#else
    (sql_stmt)
    char    *sql_stmt;
#endif

{
   int num=0;

   InternalTrace( RexxSQLData.RxPackageGlobalData, "CountQuestionMarks", "%s", sql_stmt );

   while(*sql_stmt++)
   {
      if (*sql_stmt == '?')
         num++;
   }
   return num;
}

/*-----------------------------------------------------------------------------
 * Install a REXX/SQL variable structure.
 *----------------------------------------------------------------------------*/
static SQLVAL *InstallSQLVariable

#ifdef HAVE_PROTO
    (int *rc,char *name, SQLVAL *first, int upd, int dtype, int max, int option, void *val, int valtype)
#else
    (rc,name, first, upd, dtype, max, option, val, valtype)
    int    *rc;
    char   *name;
    SQLVAL *first;
    int    upd;
    int    dtype;
    int    max;
    int    option;
    void   *val;
    int    valtype;
#endif

{
   SQLVAL  *opt=NULL;

   InternalTrace( RexxSQLData.RxPackageGlobalData, "InstallSQLVariable", "%x,%s,%x,%d,%d,%d,%d,%x,%d", rc,name, first, upd, dtype, max, option, val, valtype );

   if ((opt = (SQLVAL*)ll_add(first,first)) == (SQLVAL*)NULL)
   {
      *rc = SetIntError( __FILE__, __LINE__, 10, "out of memory");
      return(NULL);
   }
   (void)strcpy(opt->name, name);
   opt->user_update = (char)upd;
   opt->dtype = (char)dtype;
   opt->maxlen = max;
   opt->value = val;
   opt->option = option;
   opt->valtype = valtype;
   *rc = 0;
   return(opt);
}

/*-----------------------------------------------------------------------------
 * This function converts a REXX array into an array of argv's
 *----------------------------------------------------------------------------*/
static int ArrayToArgv

#ifdef HAVE_PROTO
    (RXSTRING *names, RXSTRING *values, ULONG *Argc, RXSTRING *Argv)
#else
    (names, values, Argc, Argv)
    RXSTRING *names;
    RXSTRING *values;
    ULONG *Argc;
    RXSTRING *Argv;
#endif

{
   int i=0,num_args=1;
   RXSTRING counter;
   char *new_name=NULL;
   char *new_value=NULL;
   int num_names=0,num_values=0;
  
   InternalTrace( RexxSQLData.RxPackageGlobalData, "ArrayToArgv", "%x,%x,%x,%x", names, values, Argc, Argv );
  
   if ((new_name = (char *)malloc(RXSTRLEN(*names)+1)) == NULL)
      return(SetIntError( __FILE__, __LINE__, 10, "out of memory"));
   memcpy(new_name,RXSTRPTR(*names),RXSTRLEN(*names));
   new_name[RXSTRLEN(*names)] = '\0';
   if ((GetRexxVariable( RexxSQLData.RxPackageGlobalData, new_name,&counter,0)) == NULL)
   {
      free(new_name);
      free(counter.strptr);
      return(SetIntError( __FILE__, __LINE__, 10, "rexx variable not found"));
   }
   num_names = atoi(RXSTRPTR(counter));
  
   if (values != NULL)
   {
      num_args = 2;
      if ((new_value = (char *)malloc(RXSTRLEN(*values)+1)) == NULL)
      {
         free(new_name);
         free(counter.strptr);
         return(SetIntError( __FILE__, __LINE__, 10, "out of memory"));
      }
      memcpy(new_value,RXSTRPTR(*values),RXSTRLEN(*values));
      new_value[RXSTRLEN(*values)] = '\0';
      if ((GetRexxVariable( RexxSQLData.RxPackageGlobalData, new_value,&counter,0)) == NULL)
      {
         free(new_name);
         free(counter.strptr);
         return(SetIntError( __FILE__, __LINE__, 10, "out of memory"));
      }
      num_values = atoi(RXSTRPTR(counter));
      if (num_names != num_values)
      {
         free(new_name);
         free(counter.strptr);
         return(SetIntError( __FILE__, __LINE__, 88, "array index counts not equal"));
      }
   }
   free(counter.strptr);
  
   for (i=1;i<=num_names;i++)
   {
      if ((GetRexxVariable( RexxSQLData.RxPackageGlobalData, new_name,Argv,i)) == NULL)
      {
         free(new_name);
         return(SetIntError( __FILE__, __LINE__, 10, "out of memory"));
      }
      Argv++;
      if (values != NULL)
      {
         if ((GetRexxVariable( RexxSQLData.RxPackageGlobalData, new_value,Argv,i)) == NULL)
         {
            free(new_name);
            return(SetIntError( __FILE__, __LINE__, 10, "out of memory"));
         }
         Argv++;
      }
   }
   free(new_name);
   free(new_value);
   *Argc = num_names*num_args;
   return 0;
}

/*-----------------------------------------------------------------------------
 * This function copies an array of arvs to another
 *----------------------------------------------------------------------------*/
static int ArgvToArgv

#ifdef HAVE_PROTO
    (ULONG argc, RXSTRING *argv, ULONG *Argc, RXSTRING *Argv)
#else
    (argc, argv, Argc, Argv)
    ULONG argc;
    RXSTRING *argv;
    ULONG *Argc;
    RXSTRING *Argv;
#endif

{
   ULONG i=0L;

   InternalTrace( RexxSQLData.RxPackageGlobalData, "ArgvToArgv","%d,%x,%x,%x", argc, argv, Argc, Argv );

   for (i=0;i<argc;i++)
   {
      if ((Argv[i].strptr = (char *)malloc(RXSTRLEN(argv[i])+1)) == NULL)
         return(SetIntError( __FILE__, __LINE__, 10, "out of memory"));
      memcpy(RXSTRPTR(Argv[i]),RXSTRPTR(argv[i]),RXSTRLEN(argv[i]));
      Argv[i].strptr[RXSTRLEN(argv[i])] = '\0';
      Argv[i].strlength = RXSTRLEN(argv[i]);
   }
   *Argc = argc;
   return 0;
}

/*-----------------------------------------------------------------------------
 * This function frees an array of argvs
 *----------------------------------------------------------------------------*/
static RXSTRING *FreeArgv

#ifdef HAVE_PROTO
    (ULONG *Argc, RXSTRING *Argv)
#else
    (Argc,Argv)
    ULONG *Argc;
    RXSTRING *Argv;
#endif

{
   int i=0;

   InternalTrace( RexxSQLData.RxPackageGlobalData, "FreeArgv", "%x,%x", Argc,Argv );

   if (*Argc == 0
   || Argv == NULL)
      return(NULL);

   for (i=0;i<MAX_BINDVARS*2;i++)
   {
      if (RXSTRPTR(Argv[i]) != NULL)
         free(Argv[i].strptr);
      Argv[i].strptr = NULL;
      Argv[i].strlength = 0;
   }
   *Argc = 0;
   free(Argv);
   return(NULL);
}


/*-----------------------------------------------------------------------------
 * This function is called to initiate REXX/SQL interface.
 *----------------------------------------------------------------------------*/
int InitRexxSQL

#ifdef HAVE_PROTO
    (char *progname,int regfunc)
#else
    (progname,regfunc)
    char *progname;
    int regfunc;
#endif

{
  
   return 0;
}


/*-----------------------------------------------------------------------------
 * This function is called to terminate all activity with REXX/SQL.
 *----------------------------------------------------------------------------*/
int TerminateRexxSQL

#ifdef HAVE_PROTO
    (char *progname)
#else
    (progname)
    char *progname;
#endif

{
}

/*-----------------------------------------------------------------------------
 * Add a SQLVAL structure to a linked list.
 *----------------------------------------------------------------------------*/
static SQLVAL *ll_add

#ifdef HAVE_PROTO
    (SQLVAL *first,SQLVAL *curr)
#else
    (first,curr)
    SQLVAL *first;
    SQLVAL *curr;
#endif
{
   SQLVAL *next=NULL;
   if ((next=(SQLVAL *)malloc(sizeof(SQLVAL))) != (SQLVAL *)NULL)
   {
      if (curr == NULL)
      {
         first = next;
         next->next = NULL;
      }
      else
      {
         if (curr->next != NULL)
            curr->next->prev = next;
         next->next = curr->next;
         curr->next = next;
      }
      next->prev = curr;
   }
   next->value = NULL;
   return(next);
}

/*-----------------------------------------------------------------------------
 * Free a SQLVAL linked list.
 *----------------------------------------------------------------------------*/
static SQLVAL *ll_free

#ifdef HAVE_PROTO
    (SQLVAL *first)
#else
    (first)
    SQLVAL *first;
#endif
{
   SQLVAL *curr=first;
   SQLVAL *new_curr=NULL;
   RXSTRING *rxs=NULL;
   RXSTRING_DT *rxs_dt=NULL;
   int i=0;
  
   while (curr != NULL)
   {
      if ((curr->option)&OPT_DYNAMIC)
      {
         switch(curr->dtype)
         {
            case TYPE_INT:
            case TYPE_BOOL:
               if (curr->value)
                  free(curr->value);
               break;
            case TYPE_STRING:
               rxs =(RXSTRING*)(curr->value);
               if (rxs)
               {
                  if (rxs->strptr) free(rxs->strptr);
                  free(rxs);
               }
               break;
            case TYPE_ARRAY:
               rxs =(RXSTRING*)(curr->value);
               if (rxs)
               {
                  for (i=0;;i++)
                  {
                     if (rxs[i].strlength == 0)
                        break;
                     if (rxs[i].strptr) free(rxs[i].strptr);
                  }
                  free(rxs);
               }
               break;
            case TYPE_DATATYPE:
               rxs_dt =(RXSTRING_DT*)(curr->value);
               if (rxs_dt)
               {
                  for (i=0;;i++)
                  {
                     if (rxs_dt[i].strlength == 0)
                        break;
                     if (rxs_dt[i].strptr) free(rxs_dt[i].strptr);
                  }
               }
               free(rxs_dt);
               break;
         }
      }
      new_curr = curr->next;
      free(curr);
      curr = new_curr;
   }
   return((SQLVAL *)NULL);
}

/*-----------------------------------------------------------------------------
 * Find an entry in a SQLVAL linked list.
 *----------------------------------------------------------------------------*/
static SQLVAL *ll_find

#ifdef HAVE_PROTO
    (SQLVAL *first, char *name)
#else
    (first,name)
    SQLVAL *first;
    char *name;
#endif
{
   SQLVAL *curr=NULL;
   curr = first;
   while (curr != NULL)
   {
      if (strcmp(curr->name,name) == 0)
      {
         return(curr);
      }
      curr = curr->next;
   }
   return(NULL);
}

/*-----------------------------------------------------------------------------
 * Print a usage message.
 *----------------------------------------------------------------------------*/
void RexxSQLUsage

#ifdef HAVE_PROTO
    (void)
#else
    ()
#endif

{
    (void)fprintf(stderr,
                  "\nVersion: %s %s %s %s %s\n\nUsage:   %s [-h]\n         %s [-idvDu] [REXX-script-name] [arguments...]\n\n",
                  RxPackageName,
                  REXXSQL_VERSION,
                  REXXSQL_DATE,
                  CURRENT_OS,
                  CURRENT_DB,
                  RxPackageName,
                  RxPackageName);
    exit(BAD_ARGS);
}

/*-----------------------------------------------------------------------------
 * This function is used to initialise the external functions from
 * within a dynamic library when the SysAddFunc() REXX function
 * is called.
 *----------------------------------------------------------------------------*/
#if defined(USE_REXX6000)
USHORT InitFunc(RXFUNCBLOCK **FuncBlock)
{
   static RXFUNCBLOCK funcarray[] =
   {
      { NAME_SQLCONNECT_EXT,    SQLCONNECT   ,NULL },
      { NAME_SQLDISCONNECT_EXT, SQLDISCONNECT,NULL },
      { NAME_SQLDEFAULT_EXT,    SQLDEFAULT   ,NULL },
      { NAME_SQLCOMMIT_EXT,     SQLCOMMIT    ,NULL },
      { NAME_SQLROLLBACK_EXT,   SQLROLLBACK  ,NULL },
      { NAME_SQLCOMMAND_EXT,    SQLCOMMAND   ,NULL },
      { NAME_SQLPREPARE_EXT,    SQLPREPARE   ,NULL },
      { NAME_SQLDISPOSE_EXT,    SQLDISPOSE   ,NULL },
      { NAME_SQLEXECUTE_EXT,    SQLEXECUTE   ,NULL },
      { NAME_SQLOPEN_EXT,       SQLOPEN      ,NULL },
      { NAME_SQLCLOSE_EXT,      SQLCLOSE     ,NULL },
      { NAME_SQLFETCH_EXT,      SQLFETCH     ,NULL },
      { NAME_SQLGETDATA_EXT,    SQLGETDATA   ,NULL },
      { NAME_SQLVARIABLE_EXT,   SQLVARIABLE  ,NULL },
      { NAME_SQLGETINFO_EXT,    SQLGETINFO   ,NULL },
      { NAME_SQLDESCRIBE_EXT,   SQLDESCRIBE  ,NULL },
      { NAME_SQLDROPFUNCS_EXT,  SQLDROPFUNCS ,NULL },
      { NULL,                   NULL,         NULL }
   };
   *FuncBlock = funcarray;
   return (USHORT)0;
}
#endif

int InitialisePackage

#ifdef HAVE_PROTO
   ( void )
#else
   ( )
#endif
{
   int rc = 0;

   RexxSQLData.RowLimit= 0L ;
   RexxSQLData.LongLimit = DEFAULT_LONG_LIMIT;
   RexxSQLData.SaveSQL = TRUE;
   RexxSQLData.AutoCommit = FALSE;
   RexxSQLData.IgnoreTruncate = FALSE;
   RexxSQLData.StandardPlacemarkers = FALSE;
   RexxSQLData.SupportsTransactions = SUPPORTSTRANSACTIONS;
   RexxSQLData.SupportsPlacemarkers = SUPPORTSPLACEMARKERS;
   RexxSQLData.SupportsDMLRowcount  = SUPPORTSDMLROWCOUNT;
   RexxSQLData.SupportsSqlGetData   = SUPPORTSSQLGETDATA;
   RexxSQLData.SQLCA_SqlCode = (-1L);
   RexxSQLData.SQLCA_RowCount = (-1L);
   RexxSQLData.SQLCA_IntCode = (-1L);
   rc = InitRexxSQL( RxPackageName, 0 );
   return rc;
}

#if !defined(DYNAMIC_LIBRARY) && (defined(USE_WINREXX) || defined(USE_QUERCUS))
# if defined(USE_WINREXX)
int RXSQL_APIENTRY RxExitHandlerForSayTraceRedirection
   ( int ExitNumber, int Subfunction, PEXIT ParmBlock )
# else
LONG RxExitHandlerForSayTraceRedirection
   ( LONG ExitNumber, LONG Subfunction, PEXIT ParmBlock )
# endif
{
   long i = 0;
   int rc = 0;

   switch( Subfunction )
   {
      case RXSIOSAY:
      {
         RXSIOSAY_PARM *say_parm = (RXSIOSAY_PARM *)ParmBlock;
         if ( say_parm->rxsio_string.strptr != NULL )
         {
            for( i = 0; i < (long)say_parm->rxsio_string.strlength; i++ )
               fputc( ( char )say_parm->rxsio_string.strptr[i], stdout );
         }
         fputc( '\n', stdout );
         rc = RXEXIT_HANDLED;
         break;
      }
      case RXSIOTRC:
      {
         RXSIOTRC_PARM *trc_parm = (RXSIOTRC_PARM *)ParmBlock;
         if ( trc_parm->rxsio_string.strptr != NULL )
         {
            for( i = 0; i < (long)trc_parm->rxsio_string.strlength; i++ )
               fputc( ( char )trc_parm->rxsio_string.strptr[i], stdout );
         }
         fputc( '\n', stdout );
         rc = RXEXIT_HANDLED;
         break;
      }
      case RXSIOTRD:
      {
         RXSIOTRD_PARM *trd_parm = (RXSIOTRD_PARM *)ParmBlock;
         int i = 0, ch = 0;
         do
         {
            if ( i < 256 )
               trd_parm->rxsiotrd_retc.strptr[i++] = ch = getc( stdin ) ;
         } while( ch != '\012' && (ch != EOF ) ) ;
         trd_parm->rxsiotrd_retc.strlength = i - 1;
         rc = RXEXIT_HANDLED;
         break;
      }
      case RXSIODTR:
      {
         RXSIODTR_PARM *dtr_parm = (RXSIODTR_PARM *)ParmBlock;
         int i = 0, ch = 0;
         do
         {
            if ( i < 256 )
               dtr_parm->rxsiodtr_retc.strptr[i++] = ch = getc( stdin ) ;
         } while( ch != '\012' && (ch != EOF ) ) ;
         dtr_parm->rxsiodtr_retc.strlength = i - 1;
         rc = RXEXIT_HANDLED;
         break;
      }
         break;
      default:
         rc = RXEXIT_NOT_HANDLED;
         break;
   }
  return rc;
}
#endif

/*
 * Because WinRexx and Personal Rexx load and unload the Rexx/SQL DLL
 * each time an external function is called, we need to ensure that
 * the DLL stays loaded in memory, so that memory we have initialised
 * stays set.
 * Why they do this is beyond me :-(
 */
#if defined(DYNAMIC_LIBRARY) && (defined(USE_WINREXX) || defined(USE_QUERCUS))
BOOL WINAPI DllMain( HINSTANCE hDLL, DWORD dwReason, LPVOID pReserved)
{
   switch( dwReason)
   {
      case DLL_PROCESS_ATTACH:
         LoadLibrary( RxPackageName );
         break;
      case DLL_PROCESS_DETACH:
         FreeLibrary( GetModuleHandle( RxPackageName ) );
         break;
      case DLL_THREAD_ATTACH:
         break;
      case DLL_THREAD_DETACH:
         break;
   }
   return(TRUE);
}
#endif
